/*
   Copyright (c) 2012 LinkedIn Corp.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.linkedin.d2.discovery.stores.zk;

import java.util.Collection;
import java.util.Map;

/**
 * The following class will be used to merge multiple nodes' data entries into one, and then to unmerge them
 */
public interface ZooKeeperPropertyMerger<T>
{
  /**
   * Merge multiple properties into one. The data structure T has to support this kind of operation
   */
  T merge(String propertyName, Collection<T> propertiesToMerge);

  /**
   * unmerge should return the String key of the propertiesToMerge containing the value to delete
   */
  String unmerge(String propertyName, T toDelete, Map<String, T> propertiesToMerge);
}
